@@ -18,34 +18,24 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _attach_h
 #define _attach_h
 
 #include <sys/types.h>
+#include <lxc/attach_options.h>
+
+struct lxc_conf;
 
 struct lxc_proc_context_info {
-	char *aa_profile;
+	char *lsm_label;
+	struct lxc_container *container;
 	unsigned long personality;
 	unsigned long long capability_mask;
 };
 
-extern struct lxc_proc_context_info *lxc_proc_get_context_info(pid_t pid);
-
-typedef enum lxc_attach_env_policy_t {
-	LXC_ATTACH_KEEP_ENV,
-	LXC_ATTACH_CLEAR_ENV
-} lxc_attach_env_policy_t;
-
-extern int lxc_attach_to_ns(pid_t other_pid, int which);
-extern int lxc_attach_remount_sys_proc();
-extern int lxc_attach_drop_privs(struct lxc_proc_context_info *ctx);
-extern int lxc_attach_set_environment(enum lxc_attach_env_policy_t policy, char** extra_env, char** extra_keep);
-
-extern char *lxc_attach_getpwshell(uid_t uid);
-
-extern void lxc_attach_get_init_uidgid(uid_t* init_uid, gid_t* init_gid);
+extern int lxc_attach(const char* name, const char* lxcpath, lxc_attach_exec_t exec_function, void* exec_payload, lxc_attach_options_t* options, pid_t* attached_process);
 
 #endif
