@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <stdio.h>
 #include <errno.h>
@@ -35,15 +35,30 @@
 
 #include "log.h"
 #include "caps.h"
+#include "utils.h"
 
 #define LXC_LOG_PREFIX_SIZE	32
 #define LXC_LOG_BUFFER_SIZE	512
 
+#ifdef HAVE_TLS
+__thread int lxc_log_fd = -1;
+static __thread char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
+static __thread char *log_fname = NULL;
+/* command line values for logfile or logpriority should always override
+ * values from the configuration file or defaults
+ */
+static __thread int lxc_logfile_specified = 0;
+static __thread int lxc_loglevel_specified = 0;
+#else
 int lxc_log_fd = -1;
 static char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
+static char *log_fname = NULL;
+/* command line values for logfile or logpriority should always override
+ * values from the configuration file or defaults
+ */
+static int lxc_logfile_specified = 0;
 static int lxc_loglevel_specified = 0;
-// if logfile was specifed on command line, it won't be overridden by lxc.logfile
-static int lxc_log_specified = 0;
+#endif
 
 lxc_log_define(lxc_log, lxc);
 
@@ -66,15 +81,17 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 {
 	char buffer[LXC_LOG_BUFFER_SIZE];
 	int n;
+	int ms;
 
 	if (lxc_log_fd == -1)
 		return 0;
 
+	ms = event->timestamp.tv_usec / 1000;
 	n = snprintf(buffer, sizeof(buffer),
-		     "%15s %10ld.%03ld %-8s %s - ",
+		     "%15s %10ld.%03d %-8s %s - ",
 		     log_prefix,
 		     event->timestamp.tv_sec,
-		     event->timestamp.tv_usec / 1000,
+		     ms,
 		     lxc_log_priority_to_string(event->priority),
 		     event->category);
 
@@ -119,12 +136,6 @@ struct lxc_log_category lxc_log_category_lxc = {
 };
 
 /*---------------------------------------------------------------------------*/
-extern void lxc_log_setprefix(const char *prefix)
-{
-	strncpy(log_prefix, prefix, sizeof(log_prefix));
-	log_prefix[sizeof(log_prefix) - 1] = 0;
-}
-
 static int build_dir(const char *name)
 {
 	char *n = strdup(name);  // because we'll be modifying it
@@ -180,29 +191,53 @@ static int log_open(const char *name)
 	return newfd;
 }
 
-static char *build_log_path(const char *name)
+/*
+ * Build the path to the log file
+ * @name     : the name of the container
+ * @lxcpath  : the lxcpath to use as a basename or NULL to use LOGPATH
+ * Returns malloced path on sucess, or NULL on failure
+ */
+static char *build_log_path(const char *name, const char *lxcpath)
 {
 	char *p;
-	int len, ret;
+	int len, ret, use_dir;
+
+	if (!name)
+		return NULL;
+
+#if USE_CONFIGPATH_LOGS
+	use_dir = 1;
+#else
+	use_dir = 0;
+#endif
 
 	/*
-	 * '$logpath' + '/' + '$name' + '.log' + '\0'
-	 * or
+	 * If USE_CONFIGPATH_LOGS is true or lxcpath is given, the resulting
+	 * path will be:
 	 * '$logpath' + '/' + '$name' + '/' + '$name' + '.log' + '\0'
-	 * sizeof(LOGPATH) includes its \0
+	 *
+	 * If USE_CONFIGPATH_LOGS is false the resulting path will be:
+	 * '$logpath' + '/' + '$name' + '.log' + '\0'
 	 */
-	len = sizeof(LOGPATH) + strlen(name) + 6;
-#if USE_CONFIGPATH_LOGS
-	len += strlen(name) + 1;  /* add "/$container_name/" */
-#endif
+	len = strlen(name) + 6; /* 6 == '/' + '.log' + '\0' */
+	if (lxcpath)
+		use_dir = 1;
+	else
+		lxcpath = LOGPATH;
+
+	if (use_dir)
+		len += strlen(lxcpath) + 1 + strlen(name) + 1;  /* add "/$container_name/" */
+	else
+		len += strlen(lxcpath) + 1;
 	p = malloc(len);
 	if (!p)
 		return p;
-#if USE_CONFIGPATH_LOGS
-	ret = snprintf(p, len, "%s/%s/%s.log", LOGPATH, name, name);
-#else
-	ret = snprintf(p, len, "%s/%s.log", LOGPATH, name);
-#endif
+
+	if (use_dir)
+		ret = snprintf(p, len, "%s/%s/%s.log", lxcpath, name, name);
+	else
+		ret = snprintf(p, len, "%s/%s.log", lxcpath, name);
+
 	if (ret < 0 || ret >= len) {
 		free(p);
 		return NULL;
@@ -210,30 +245,76 @@ static char *build_log_path(const char *name)
 	return p;
 }
 
-int do_lxc_log_set_file(const char *fname, int from_default);
+/*
+ * This can be called:
+ *   1. when a program calls lxc_log_init with no logfile parameter (in which
+ *      case the default is used).  In this case lxc.logfile can override this.
+ *   2. when a program calls lxc_log_init with a logfile parameter.  In this
+ *	case we don't want lxc.logfile to override this.
+ *   3. When a lxc.logfile entry is found in config file.
+ */
+static int __lxc_log_set_file(const char *fname, int create_dirs)
+{
+	if (lxc_log_fd != -1) {
+		// we are overriding the default.
+		close(lxc_log_fd);
+		free(log_fname);
+	}
+
+	if (!fname || strlen(fname) == 0) {
+		log_fname = NULL;
+		return 0;
+	}
+
+#if USE_CONFIGPATH_LOGS
+	// we don't build_dir for the default if the default is
+	// i.e. /var/lib/lxc/$container/$container.log
+	if (create_dirs)
+#endif
+	if (build_dir(fname)) {
+		ERROR("failed to create dir for log file \"%s\" : %s", fname,
+		      strerror(errno));
+		return -1;
+	}
+
+	lxc_log_fd = log_open(fname);
+	if (lxc_log_fd == -1)
+		return -1;
+
+	log_fname = strdup(fname);
+	return 0;
+}
+
+static int _lxc_log_set_file(const char *name, const char *lxcpath, int create_dirs)
+{
+	char *logfile;
+	int ret;
+
+	logfile = build_log_path(name, lxcpath);
+	if (!logfile) {
+		ERROR("could not build log path");
+		return -1;
+	}
+	ret = __lxc_log_set_file(logfile, create_dirs);
+	free(logfile);
+	return ret;
+}
 
-/*---------------------------------------------------------------------------*/
 extern int lxc_log_init(const char *name, const char *file,
-			const char *priority, const char *prefix, int quiet)
+			const char *priority, const char *prefix, int quiet,
+			const char *lxcpath)
 {
 	int lxc_priority = LXC_LOG_PRIORITY_ERROR;
 	int ret;
-	char *tmpfile = NULL;
-	int want_lxc_log_specified = 0;
 
-	if (lxc_log_fd != -1)
+	if (lxc_log_fd != -1) {
+		WARN("lxc_log_init called with log already initialized");
 		return 0;
-
-	if (priority) {
-		lxc_loglevel_specified = 1;
-		lxc_priority = lxc_log_priority_to_int(priority);
-
-		if (lxc_priority == LXC_LOG_PRIORITY_NOTSET) {
-			ERROR("invalid log priority %s", priority);
-			return -1;
-		}
 	}
 
+	if (priority)
+		lxc_priority = lxc_log_priority_to_int(priority);
+
 	lxc_log_category_lxc.priority = lxc_priority;
 	lxc_log_category_lxc.appender = &log_appender_logfile;
 
@@ -241,42 +322,62 @@ extern int lxc_log_init(const char *name, const char *file,
 		lxc_log_category_lxc.appender->next = &log_appender_stderr;
 
 	if (prefix)
-		lxc_log_setprefix(prefix);
+		lxc_log_set_prefix(prefix);
 
-	if (file && strcmp(file, "none") == 0) {
-		want_lxc_log_specified = 1;
-		return 0;
-	}
-
-	if (!file) {
-		tmpfile = build_log_path(name);
-		if (!tmpfile) {
-			ERROR("could not build log path");
-			return -1;
-		}
+	if (file) {
+		if (strcmp(file, "none") == 0)
+			return 0;
+		ret = __lxc_log_set_file(file, 1);
 	} else {
-		want_lxc_log_specified = 1;
+
+		/* For now, unprivileged containers have to set -l to get logging */
+		if (geteuid())
+			return 0;
+
+		/* if no name was specified, there nothing to do */
+		if (!name)
+			return 0;
+
+		ret = -1;
+
+		if (!lxcpath)
+			lxcpath = LOGPATH;
+
+		/* try LOGPATH if lxcpath is the default */
+		if (strcmp(lxcpath, lxc_global_config_value("lxc.lxcpath")) == 0)
+			ret = _lxc_log_set_file(name, NULL, 0);
+
+		/* try in lxcpath */
+		if (ret < 0)
+			ret = _lxc_log_set_file(name, lxcpath, 1);
+
+		/* try LOGPATH in case its writable by the caller */
+		if (ret < 0)
+			ret = _lxc_log_set_file(name, NULL, 0);
 	}
 
-	ret = do_lxc_log_set_file(tmpfile ? tmpfile : file, !want_lxc_log_specified);
-
-	if (want_lxc_log_specified)
-		lxc_log_specified = 1;
 	/*
-	 * If !want_lxc_log_specified, that is, if the user did not request
-	 * this logpath, then ignore failures and continue logging to console
+	 * If !file, that is, if the user did not request this logpath, then
+	 * ignore failures and continue logging to console
 	 */
-	if (!want_lxc_log_specified && ret != 0) {
+	if (!file && ret != 0) {
 		INFO("Ignoring failure to open default logfile.");
 		ret = 0;
 	}
 
-	if (tmpfile)
-		free(tmpfile);
-
 	return ret;
 }
 
+extern void lxc_log_close(void)
+{
+	if (lxc_log_fd == -1)
+		return;
+	close(lxc_log_fd);
+	lxc_log_fd = -1;
+	free(log_fname);
+	log_fname = NULL;
+}
+
 /*
  * This is called when we read a lxc.loglevel entry in a lxc.conf file.  This
  * happens after processing command line arguments, which override the .conf
@@ -294,59 +395,52 @@ extern int lxc_log_set_level(int level)
 	return 0;
 }
 
-char *log_fname;  // default to NULL, set in lxc_log_set_file.
-/*
- * This can be called:
- *   1. when a program calls lxc_log_init with no logfile parameter (in which
- *      case the default is used).  In this case lxc.logfile can override this.
- *   2. when a program calls lxc_log_init with a logfile parameter.  In this
- *	case we don't want lxc.logfile to override this.
- *   3. When a lxc.logfile entry is found in config file.
- */
-int do_lxc_log_set_file(const char *fname, int from_default)
-{
-	if (lxc_log_specified) {
-		INFO("lxc.logfile overridden by command line");
-		return 0;
-	}
-	if (lxc_log_fd != -1) {
-		// we are overriding the default.
-		close(lxc_log_fd);
-		free(log_fname);
-	}
-
-#if USE_CONFIGPATH_LOGS
-	// we don't build_dir for the default if the default is
-	// i.e. /var/lib/lxc/$container/$container.log
-	if (!from_default)
-#endif
-	if (build_dir(fname)) {
-		ERROR("failed to create dir for log file \"%s\" : %s", fname,
-		      strerror(errno));
-		return -1;
-	}
-
-	lxc_log_fd = log_open(fname);
-	if (lxc_log_fd == -1)
-		return -1;
-
-	log_fname = strdup(fname);
-	return 0;
-}
-
-extern int lxc_log_set_file(const char *fname)
-{
-	return do_lxc_log_set_file(fname, 0);
-}
-
 extern int lxc_log_get_level(void)
 {
-	if (!lxc_loglevel_specified)
-		return LXC_LOG_PRIORITY_NOTSET;
 	return lxc_log_category_lxc.priority;
 }
 
+extern bool lxc_log_has_valid_level(void)
+{
+	int log_level = lxc_log_get_level();
+	if (log_level < 0 || log_level >= LXC_LOG_PRIORITY_NOTSET)
+		return false;
+	return true;
+}
+
+/*
+ * This is called when we read a lxc.logfile entry in a lxc.conf file.  This
+ * happens after processing command line arguments, which override the .conf
+ * settings.  So only set the file if previously unset.
+ */
+extern int lxc_log_set_file(const char *fname)
+{
+	if (lxc_logfile_specified)
+		return 0;
+	return __lxc_log_set_file(fname, 0);
+}
+
 extern const char *lxc_log_get_file(void)
 {
 	return log_fname;
 }
+
+extern void lxc_log_set_prefix(const char *prefix)
+{
+	strncpy(log_prefix, prefix, sizeof(log_prefix));
+	log_prefix[sizeof(log_prefix) - 1] = 0;
+}
+
+extern const char *lxc_log_get_prefix(void)
+{
+	return log_prefix;
+}
+
+extern void lxc_log_options_no_override()
+{
+	if (lxc_log_get_file())
+		lxc_logfile_specified = 1;
+
+	if (lxc_log_get_level() != LXC_LOG_PRIORITY_NOTSET)
+		lxc_loglevel_specified = 1;
+}
