@@ -19,15 +19,19 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef _log_h
 #define _log_h
 
+#include "config.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <sys/time.h>
 #include <string.h>
+#include <strings.h>
+#include <stdbool.h>
 
 #ifndef O_CLOEXEC
 #define O_CLOEXEC 02000000
@@ -172,7 +176,7 @@ __lxc_log(const struct lxc_log_category* category,
 }
 
 /*
- * Helper macro to define log fonctions.
+ * Helper macro to define log functions.
  */
 #define lxc_log_priority_define(acategory, PRIORITY)			\
 									\
@@ -232,8 +236,6 @@ static inline void LXC_##PRIORITY(struct lxc_log_locinfo* locinfo,	\
 /*
  * top categories
  */
-extern struct lxc_log_category lxc_log_category_lxc;
-
 #define TRACE(format, ...) do {						\
 	struct lxc_log_locinfo locinfo = LXC_LOG_LOCINFO_INIT;		\
 	LXC_TRACE(&locinfo, format, ##__VA_ARGS__);			\
@@ -285,14 +287,22 @@ extern struct lxc_log_category lxc_log_category_lxc;
 	ERROR("%s - " format, strerror(errno), ##__VA_ARGS__);		\
 } while (0)
 
+#ifdef HAVE_TLS
+extern __thread int lxc_log_fd;
+#else
 extern int lxc_log_fd;
+#endif
 
 extern int lxc_log_init(const char *name, const char *file,
-			const char *priority, const char *prefix, int quiet);
+			const char *priority, const char *prefix, int quiet,
+			const char *lxcpath);
 
-extern void lxc_log_setprefix(const char *a_prefix);
-extern int lxc_log_set_level(int level);
 extern int lxc_log_set_file(const char *fname);
-extern int lxc_log_get_level(void);
+extern int lxc_log_set_level(int level);
+extern void lxc_log_set_prefix(const char *prefix);
 extern const char *lxc_log_get_file(void);
+extern int lxc_log_get_level(void);
+extern bool lxc_log_has_valid_level(void);
+extern const char *lxc_log_get_prefix(void);
+extern void lxc_log_options_no_override();
 #endif
