@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __lxc_h
 #define __lxc_h
@@ -28,7 +28,9 @@ extern "C" {
 #endif
 
 #include <stddef.h>
-#include <lxc/state.h>
+#include <sys/select.h>
+#include <sys/types.h>
+#include "state.h"
 
 struct lxc_msg;
 struct lxc_conf;
@@ -50,14 +52,6 @@ struct lxc_arguments;
 extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf,
 		     const char *lxcpath);
 
-/*
- * Stop the container previously started with lxc_start, all
- * the processes running inside this container will be killed.
- * @name : the name of the container
- * Returns 0 on success, < 0 otherwise
- */
-extern int lxc_stop(const char *name, const char *lxcpath);
-
 /*
  * Start the specified command inside an application container
  * @name     : the name of the container
@@ -77,16 +71,36 @@ extern int lxc_execute(const char *name, char *const argv[], int quiet,
 extern int lxc_monitor_open(const char *lxcpath);
 
 /*
- * Read the state of the container if this one has changed
- * The function will block until there is an event available
- * @fd : the file descriptor provided by lxc_monitor_open
- * @state : the variable which will be filled with the state
+ * Blocking read for the next container state change
+ * @fd  : the file descriptor provided by lxc_monitor_open
+ * @msg : the variable which will be filled with the state
  * Returns 0 if the monitored container has exited, > 0 if
- * data was readen, < 0 otherwise
+ * data was read, < 0 otherwise
  */
 extern int lxc_monitor_read(int fd, struct lxc_msg *msg);
+
+/*
+ * Blocking read for the next container state change with timeout
+ * @fd      : the file descriptor provided by lxc_monitor_open
+ * @msg     : the variable which will be filled with the state
+ * @timeout : the timeout in seconds to wait for a state change
+ * Returns 0 if the monitored container has exited, > 0 if
+ * data was read, < 0 otherwise
+ */
 extern int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout);
 
+/*
+ * Blocking read from multiple monitors for the next container state
+ * change with timeout
+ * @rfds    : an fd_set of file descriptors provided by lxc_monitor_open
+ * @nfds    : the maximum fd number in rfds + 1
+ * @msg     : the variable which will be filled with the state
+ * @timeout : the timeout in seconds to wait for a state change
+ * Returns 0 if the monitored container has exited, > 0 if
+ * data was read, < 0 otherwise
+ */
+extern int lxc_monitor_read_fdset(fd_set *rfds, int nfds, struct lxc_msg *msg, int timeout);
+
 /*
  * Close the fd associated with the monitoring
  * @fd : the file descriptor provided by lxc_monitor_open
@@ -94,15 +108,6 @@ extern int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout);
  */
 extern int lxc_monitor_close(int fd);
 
-/*
- * Show the console of the container.
- * @name : the name of container
- * @tty  : the tty number
- * @fd   : a pointer to a tty file descriptor
- * Returns 0 on sucess, < 0 otherwise
- */
-extern int lxc_console(const char *name, int ttynum, int *fd, const char *lxcpath);
-
 /*
  * Freeze all the tasks running inside the container <name>
  * @name : the container name
@@ -117,14 +122,6 @@ extern int lxc_freeze(const char *name, const char *lxcpath);
  */
 extern int lxc_unfreeze(const char *name, const char *lxcpath);
 
-/*
- * Unfreeze all previously frozen tasks.
- * This is the function to use from inside the monitor
- * @name : the name of the container
- * Return 0 on sucess, < 0 otherwise
- */
-extern int lxc_unfreeze_bypath(const char *cgpath);
-
 /*
  * Retrieve the container state
  * @name : the name of the container
@@ -135,36 +132,25 @@ extern lxc_state_t lxc_state(const char *name, const char *lxcpath);
 /*
  * Set a specified value for a specified subsystem. The specified
  * subsystem must be fully specified, eg. "cpu.shares"
- * @cgpath    : the cgroup path of the container
  * @filename  : the cgroup attribute filename
  * @value     : the value to be set
- * Returns 0 on success, < 0 otherwise
- */
-extern int lxc_cgroup_set_bypath(const char *cgpath, const char *filename, const char *value);
-
-/*
- * Set a specified value for a specified subsystem. The specified
- * subsystem must be fully specified, eg. "cpu.shares"
  * @name      : the name of the container
- * @filename  : the cgroup attribute filename
- * @value     : the value to be set
  * @lxcpath   : lxc config path for container
  * Returns 0 on success, < 0 otherwise
  */
-extern int lxc_cgroup_set(const char *name, const char *filename, const char *value, const char *lxcpath);
+extern int lxc_cgroup_set(const char *filename, const char *value, const char *name, const char *lxcpath);
 
 /*
  * Get a specified value for a specified subsystem. The specified
  * subsystem must be fully specified, eg. "cpu.shares"
- * @name      : the name of the container
  * @filename  : the cgroup attribute filename
  * @value     : the value to be set
  * @len       : the len of the value variable
+ * @name      : the name of the container
  * @lxcpath   : lxc config path for container
  * Returns the number of bytes read, < 0 on error
  */
-extern int lxc_cgroup_get(const char *name, const char *filename,
-			  char *value, size_t len, const char *lxcpath);
+extern int lxc_cgroup_get(const char *filename, char *value, size_t len, const char *name, const char *lxcpath);
 
 /*
  * Retrieve the error string associated with the error returned by
@@ -174,28 +160,6 @@ extern int lxc_cgroup_get(const char *name, const char *filename,
  */
 extern const char *lxc_strerror(int error);
 
-/*
- * Checkpoint a container
- * @name : the name of the container being checkpointed
- * @sfd: fd on which the container is checkpointed
- * @flags : checkpoint flags (an ORed value)
- * Returns 0 on success, < 0 otherwise
- */
-extern int lxc_checkpoint(const char *name, int sfd, int flags);
-#define LXC_FLAG_PAUSE 1
-#define LXC_FLAG_HALT  2
-
-/*
- * Restart a container
- * @name : the name of the container being restarted
- * @sfd: fd from which the container is restarted
- * @conf: lxc_conf structure.
- * @flags : restart flags (an ORed value)
- * @lxcpath: container path
- * Returns 0 on success, < 0 otherwise
- */
-extern int lxc_restart(const char *, int, struct lxc_conf *, int, const char *);
-
 /*
  * Create and return a new lxccontainer struct.
  */
@@ -220,6 +184,11 @@ extern int lxc_container_put(struct lxc_container *c);
  */
 extern int lxc_get_wait_states(const char **states);
 
+/*
+ * Add a dependency to a container
+ */
+extern int add_rdepend(struct lxc_conf *lxc_conf, char *rdepend);
+
 #ifdef __cplusplus
 }
 #endif
