@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <stdio.h>
 #include <unistd.h>
@@ -26,11 +26,15 @@
 #include <libgen.h>
 #include <string.h>
 
-#include <lxc/lxc.h>
-#include <lxc/log.h>
+#include <lxc/lxccontainer.h>
+
+#include "lxc.h"
+#include "log.h"
 
 #include "arguments.h"
 
+lxc_log_define(lxc_freeze_ui, lxc);
+
 static const struct option my_longopts[] = {
 	LXC_COMMON_OPTIONS
 };
@@ -51,13 +55,38 @@ Options :\n\
 
 int main(int argc, char *argv[])
 {
+	struct lxc_container *c;
+
 	if (lxc_arguments_parse(&my_args, argc, argv))
-		return -1;
+		exit(1);
+
+	if (!my_args.log_file)
+		my_args.log_file = "none";
 
 	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
-			 my_args.progname, my_args.quiet))
-		return -1;
+			 my_args.progname, my_args.quiet, my_args.lxcpath[0]))
+		exit(1);
+	lxc_log_options_no_override();
 
-	return lxc_freeze(my_args.name, my_args.lxcpath);
+	c = lxc_container_new(my_args.name, my_args.lxcpath[0]);
+	if (!c) {
+		ERROR("No such container: %s:%s", my_args.lxcpath[0], my_args.name);
+		exit(1);
+	}
+
+	if (!c->may_control(c)) {
+		ERROR("Insufficent privileges to control %s:%s", my_args.lxcpath[0], my_args.name);
+		lxc_container_put(c);
+		exit(1);
+	}
+
+	if (!c->freeze(c)) {
+		ERROR("Failed to freeze %s:%s", my_args.lxcpath[0], my_args.name);
+		lxc_container_put(c);
+		exit(1);
+	}
+
+	lxc_container_put(c);
+
+	exit(0);
 }
-
