@@ -18,16 +18,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#define _GNU_SOURCE
+#include "config.h"
+
 #include <stdio.h>
-#undef _GNU_SOURCE
 #include <libgen.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <termios.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -40,17 +39,22 @@
 #include <netinet/in.h>
 #include <net/if.h>
 
+#include <lxc/lxccontainer.h>
+
 #include "log.h"
 #include "caps.h"
 #include "lxc.h"
 #include "conf.h"
 #include "cgroup.h"
 #include "utils.h"
-#include "config.h"
 #include "confile.h"
 #include "arguments.h"
 
-lxc_log_define(lxc_start_ui, lxc_start);
+#define OPT_SHARE_NET OPT_USAGE+1
+#define OPT_SHARE_IPC OPT_USAGE+2
+#define OPT_SHARE_UTS OPT_USAGE+3
+
+lxc_log_define(lxc_start_ui, lxc);
 
 static struct lxc_list defines;
 
@@ -62,11 +66,12 @@ static int ensure_path(char **confpath, const char *path)
 	if (path) {
 		if (access(path, W_OK)) {
 			fd = creat(path, 0600);
-			if (fd < 0) {
+			if (fd < 0 && errno != EEXIST) {
 				SYSERROR("failed to create '%s'", path);
 				goto err;
 			}
-			close(fd);
+			if (fd >= 0)
+				close(fd);
 		}
 
 		fullpath = realpath(path, NULL);
@@ -89,16 +94,66 @@ err:
 	return err;
 }
 
+static int pid_from_lxcname(const char *lxcname_or_pid, const char *lxcpath) {
+	char *eptr;
+	int pid = strtol(lxcname_or_pid, &eptr, 10);
+	if (*eptr != '\0' || pid < 1) {
+		struct lxc_container *s;
+		s = lxc_container_new(lxcname_or_pid, lxcpath);
+		if (!s) {
+			SYSERROR("'%s' is not a valid pid nor a container name", lxcname_or_pid);
+			return -1;
+		}
+
+		if (!s->may_control(s)) {
+			SYSERROR("Insufficient privileges to control container '%s'", s->name);
+			lxc_container_put(s);
+			return -1;
+		}
+
+		pid = s->init_pid(s);
+		if (pid < 1) {
+			SYSERROR("Is container '%s' running?", s->name);
+			lxc_container_put(s);
+			return -1;
+		}
+
+		lxc_container_put(s);
+	}
+	if (kill(pid, 0) < 0) {
+		SYSERROR("Can't send signal to pid %d", pid);
+		return -1;
+	}
+
+	return pid;
+}
+
+static int open_ns(int pid, const char *ns_proc_name) {
+	int fd;
+	char path[MAXPATHLEN];
+	snprintf(path, MAXPATHLEN, "/proc/%d/ns/%s", pid, ns_proc_name);
+
+	fd = open(path, O_RDONLY);
+	if (fd < 0) {
+		SYSERROR("failed to open %s", path);
+		return -1;
+	}
+	return fd;
+}
+
 static int my_parser(struct lxc_arguments* args, int c, char* arg)
 {
 	switch (c) {
 	case 'c': args->console = arg; break;
 	case 'L': args->console_log = arg; break;
-	case 'd': args->daemonize = 1; args->close_all_fds = 1; break;
+	case 'd': args->daemonize = 1; break;
 	case 'f': args->rcfile = arg; break;
 	case 'C': args->close_all_fds = 1; break;
 	case 's': return lxc_config_define_add(&defines, arg);
 	case 'p': args->pidfile = arg; break;
+	case OPT_SHARE_NET: args->share_ns[LXC_NS_NET] = arg; break;
+	case OPT_SHARE_IPC: args->share_ns[LXC_NS_IPC] = arg; break;
+	case OPT_SHARE_UTS: args->share_ns[LXC_NS_UTS] = arg; break;
 	}
 	return 0;
 }
@@ -111,6 +166,9 @@ static const struct option my_longopts[] = {
 	{"console-log", required_argument, 0, 'L'},
 	{"close-all-fds", no_argument, 0, 'C'},
 	{"pidfile", required_argument, 0, 'p'},
+	{"share-net", required_argument, 0, OPT_SHARE_NET},
+	{"share-ipc", required_argument, 0, OPT_SHARE_IPC},
+	{"share-uts", required_argument, 0, OPT_SHARE_UTS},
 	LXC_COMMON_OPTIONS
 };
 
@@ -131,7 +189,9 @@ Options :\n\
   -C, --close-all-fds    If any fds are inherited, close them\n\
                          If not specified, exit with failure instead\n\
 		         Note: --daemon implies --close-all-fds\n\
-  -s, --define KEY=VAL   Assign VAL to configuration variable KEY\n",
+  -s, --define KEY=VAL   Assign VAL to configuration variable KEY\n\
+      --share-[net|ipc|uts]=NAME Share a namespace with another container or pid\n\
+",
 	.options   = my_longopts,
 	.parser    = my_parser,
 	.checker   = NULL,
@@ -141,7 +201,7 @@ Options :\n\
 
 int main(int argc, char *argv[])
 {
-	int err = -1;
+	int err = 1;
 	struct lxc_conf *conf;
 	char *const *args;
 	char *rcfile = NULL;
@@ -149,7 +209,7 @@ int main(int argc, char *argv[])
 		"/sbin/init",
 		'\0',
 	};
-	FILE *pid_fp = NULL;
+	struct lxc_container *c;
 
 	lxc_list_init(&defines);
 
@@ -165,16 +225,36 @@ int main(int argc, char *argv[])
 		args = my_args.argv;
 
 	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
-			 my_args.progname, my_args.quiet))
+			 my_args.progname, my_args.quiet, my_args.lxcpath[0]))
 		return err;
+	lxc_log_options_no_override();
 
+	const char *lxcpath = my_args.lxcpath[0];
+
+	/*
+	 * rcfile possibilities:
+	 * 1. rcfile from random path specified in cli option
+	 * 2. rcfile not specified, use $lxcpath/$lxcname/config
+	 * 3. rcfile not specified and does not exist.
+	 */
 	/* rcfile is specified in the cli option */
-	if (my_args.rcfile)
+	if (my_args.rcfile) {
 		rcfile = (char *)my_args.rcfile;
-	else {
+		c = lxc_container_new(my_args.name, lxcpath);
+		if (!c) {
+			ERROR("Failed to create lxc_container");
+			return err;
+		}
+		c->clear_config(c);
+		if (!c->load_config(c, rcfile)) {
+			ERROR("Failed to load rcfile");
+			lxc_container_put(c);
+			return err;
+		}
+	} else {
 		int rc;
 
-		rc = asprintf(&rcfile, "%s/%s/config", my_args.lxcpath, my_args.name);
+		rc = asprintf(&rcfile, "%s/%s/config", lxcpath, my_args.name);
 		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return err;
@@ -186,88 +266,71 @@ int main(int argc, char *argv[])
 			free(rcfile);
 			rcfile = NULL;
 		}
+		c = lxc_container_new(my_args.name, lxcpath);
+		if (!c) {
+			ERROR("Failed to create lxc_container");
+			return err;
+		}
 	}
 
-	conf = lxc_conf_init();
-	if (!conf) {
-		ERROR("failed to initialize configuration");
-		return err;
-	}
-
-	if (rcfile && lxc_config_read(rcfile, conf)) {
-		ERROR("failed to read configuration file");
-		return err;
-	}
+	/*
+	 * We should use set_config_item() over &defines, which would handle
+	 * unset c->lxc_conf for us and let us not use lxc_config_define_load()
+	 */
+	if (!c->lxc_conf)
+		c->lxc_conf = lxc_conf_init();
+	conf = c->lxc_conf;
 
 	if (lxc_config_define_load(&defines, conf))
-		return err;
+		goto out;
 
 	if (!rcfile && !strcmp("/sbin/init", args[0])) {
-		ERROR("no configuration file for '/sbin/init' (may crash the host)");
-		return err;
+		ERROR("Executing '/sbin/init' with no configuration file may crash the host");
+		goto out;
 	}
 
 	if (ensure_path(&conf->console.path, my_args.console) < 0) {
 		ERROR("failed to ensure console path '%s'", my_args.console);
-		return err;
+		goto out;
 	}
 
 	if (ensure_path(&conf->console.log_path, my_args.console_log) < 0) {
 		ERROR("failed to ensure console log '%s'", my_args.console_log);
-		return err;
+		goto out;
 	}
 
 	if (my_args.pidfile != NULL) {
-		pid_fp = fopen(my_args.pidfile, "w");
-		if (pid_fp == NULL) {
-			SYSERROR("failed to create pidfile '%s' for '%s'",
-				 my_args.pidfile, my_args.name);
-			return err;
+		if (ensure_path(&c->pidfile, my_args.pidfile) < 0) {
+			ERROR("failed to ensure pidfile '%s'", my_args.pidfile);
+			goto out;
 		}
 	}
 
-	if (my_args.daemonize) {
-		/* do an early check for needed privs, since otherwise the
-		 * user won't see the error */
+	int i;
+	for (i = 0; i < LXC_NS_MAX; i++) {
+		if (my_args.share_ns[i] == NULL)
+			continue;
 
-		if (!lxc_caps_check()) {
-			ERROR("Not running with sufficient privilege");
-			return err;
-		}
+		int pid = pid_from_lxcname(my_args.share_ns[i], lxcpath);
+		if (pid < 1)
+			goto out;
 
-		if (daemon(0, 0)) {
-			SYSERROR("failed to daemonize '%s'", my_args.name);
-			return err;
-		}
+		int fd = open_ns(pid, ns_info[i].proc_name);
+		if (fd < 0)
+			goto out;
+		conf->inherit_ns_fd[i] = fd;
 	}
 
-	if (pid_fp != NULL) {
-		if (fprintf(pid_fp, "%d\n", getpid()) < 0) {
-			SYSERROR("failed to write '%s'", my_args.pidfile);
-			return err;
-		}
-		fclose(pid_fp);
+	if (!my_args.daemonize) {
+		c->want_daemonize(c, false);
 	}
 
 	if (my_args.close_all_fds)
-		conf->close_all_fds = 1;
-
-	err = lxc_start(my_args.name, args, conf, my_args.lxcpath);
-
-	/*
-	 * exec ourself, that requires to have all opened fd
-	 * with the close-on-exec flag set
-	 */
-	if (conf->reboot) {
-		INFO("rebooting container");
-		execvp(argv[0], argv);
-		SYSERROR("failed to exec");
-		err = -1;
-	}
-
-	if (my_args.pidfile)
-		unlink(my_args.pidfile);
+		c->want_close_all_fds(c, true);
 
+	err = c->start(c, 0, args) ? 0 : 1;
+out:
+	lxc_container_put(c);
 	return err;
 }
 
