@@ -18,69 +18,95 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#define _GNU_SOURCE
+#include "config.h"
+
 #include <stdio.h>
-#undef _GNU_SOURCE
-#include <stdlib.h>
-#include <unistd.h>
-#include <libgen.h>
-#include <getopt.h>
-#include <string.h>
-#include <signal.h>
 #include <errno.h>
+#include <getopt.h>
+#include <libgen.h>
+#include <netinet/in.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/syscall.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <pwd.h>
+#include <unistd.h>
 
 #include "caps.h"
-#include "log.h"
-#include "namespace.h"
 #include "cgroup.h"
 #include "error.h"
+#include "log.h"
+#include "namespace.h"
+#include "network.h"
+#include "utils.h"
+
+/* Define sethostname() if missing from the C library */
+#ifndef HAVE_SETHOSTNAME
+static int sethostname(const char * name, size_t len)
+{
+#ifdef __NR_sethostname
+return syscall(__NR_sethostname, name, len);
+#else
+errno = ENOSYS;
+return -1;
+#endif
+}
+#endif
 
 lxc_log_define(lxc_unshare_ui, lxc);
 
-void usage(char *cmd)
+struct my_iflist
+{
+	char *mi_ifname;
+	struct my_iflist *mi_next;
+};
+
+static void usage(char *cmd)
 {
 	fprintf(stderr, "%s <options> command [command_arguments]\n", basename(cmd));
 	fprintf(stderr, "Options are:\n");
-	fprintf(stderr, "\t -s flags: ORed list of flags to unshare:\n" \
+	fprintf(stderr, "\t -s flags   : ORed list of flags to unshare:\n" \
 			"\t           MOUNT, PID, UTSNAME, IPC, USER, NETWORK\n");
-	fprintf(stderr, "\t -u <id> : new id to be set if -s USER is specified\n");
+	fprintf(stderr, "\t -u <id>      : new id to be set if -s USER is specified\n");
+	fprintf(stderr, "\t -i <iface>   : Interface name to be moved into container (presumably with NETWORK unsharing set)\n");
+	fprintf(stderr, "\t -H <hostname>: Set the hostname in the container\n");
+	fprintf(stderr, "\t -d           : Daemonize (do not wait for container to exit)\n");
+	fprintf(stderr, "\t -M           : reMount default fs inside container (/proc /dev/shm /dev/mqueue)\n");
 	_exit(1);
 }
 
-static uid_t lookup_user(const char *optarg)
+static bool lookup_user(const char *optarg, uid_t *uid)
 {
 	char name[sysconf(_SC_LOGIN_NAME_MAX)];
-	uid_t uid = -1;
 	struct passwd *pwent = NULL;
 
 	if (!optarg || (optarg[0] == '\0'))
-		return uid;
+		return false;
 
-	if (sscanf(optarg, "%u", &uid) < 1) {
+	if (sscanf(optarg, "%u", uid) < 1) {
 		/* not a uid -- perhaps a username */
 		if (sscanf(optarg, "%s", name) < 1)
-			return uid;
+			return false;
 
 		pwent = getpwnam(name);
 		if (!pwent) {
 			ERROR("invalid username %s", name);
-			return uid;
+			return false;
 		}
-		uid = pwent->pw_uid;
+		*uid = pwent->pw_uid;
 	} else {
-		pwent = getpwuid(uid);
+		pwent = getpwuid(*uid);
 		if (!pwent) {
-			ERROR("invalid uid %d", uid);
-			uid = -1;
-			return uid;
+			ERROR("invalid uid %u", *uid);
+			return false;
 		}
 	}
-	return uid;
+	return true;
 }
 
 
@@ -88,6 +114,9 @@ struct start_arg {
 	char ***args;
 	int *flags;
 	uid_t *uid;
+	bool setuid;
+        int want_default_mounts;
+        const char *want_hostname;
 };
 
 static int do_start(void *arg)
@@ -96,8 +125,20 @@ static int do_start(void *arg)
 	char **args = *start_arg->args;
 	int flags = *start_arg->flags;
 	uid_t uid = *start_arg->uid;
+	int want_default_mounts = start_arg->want_default_mounts;
+	const char *want_hostname = start_arg->want_hostname;
 
-	if (flags & CLONE_NEWUSER && setuid(uid)) {
+	if ((flags & CLONE_NEWNS) && want_default_mounts)
+		lxc_setup_fs();
+
+	if ((flags & CLONE_NEWUTS) && want_hostname)
+		if (sethostname(want_hostname, strlen(want_hostname)) < 0) {
+			ERROR("failed to set hostname %s: %s", want_hostname, strerror(errno));
+			exit(1);
+		}
+
+	// Setuid is useful even without a new user id space
+	if (start_arg->setuid && setuid(uid)) {
 		ERROR("failed to set uid %d: %s", uid, strerror(errno));
 		exit(1);
 	}
@@ -115,26 +156,49 @@ int main(int argc, char *argv[])
 	char *namespaces = NULL;
 	char **args;
 	int flags = 0;
-	uid_t uid = -1; /* valid only if (flags & CLONE_NEWUSER) */
+	int daemonize = 0;
+	uid_t uid = 0; /* valid only if (flags & CLONE_NEWUSER) */
 	pid_t pid;
-
+	struct my_iflist *tmpif, *my_iflist = NULL;
 	struct start_arg start_arg = {
 		.args = &args,
 		.uid = &uid,
+		.setuid = false,
 		.flags = &flags,
+		.want_hostname = NULL,
+		.want_default_mounts = 0,
 	};
 
-	while ((opt = getopt(argc, argv, "s:u:h")) != -1) {
+	while ((opt = getopt(argc, argv, "s:u:hH:i:dM")) != -1) {
 		switch (opt) {
 		case 's':
 			namespaces = optarg;
 			break;
+		case 'i':
+			if (!(tmpif = malloc(sizeof(*tmpif)))) {
+				perror("malloc");
+				exit(1);
+			}
+			tmpif->mi_ifname = optarg;
+			tmpif->mi_next = my_iflist;
+			my_iflist = tmpif;
+			break;
+		case 'd':
+			daemonize = 1;
+			break;
+		case 'M':
+			start_arg.want_default_mounts = 1;
+			break;
+		case 'H':
+			start_arg.want_hostname = optarg;
+			break;
 		case 'h':
 			usage(argv[0]);
+			break;
 		case 'u':
-			uid = lookup_user(optarg);
-			if (uid == -1)
+			if (!lookup_user(optarg, &uid))
 				return 1;
+			start_arg.setuid = true;
 		}
 	}
 
@@ -147,26 +211,46 @@ int main(int argc, char *argv[])
 
 	ret = lxc_caps_init();
 	if (ret)
-		return ret;
+		return 1;
 
 	ret = lxc_fill_namespace_flags(namespaces, &flags);
 	if (ret)
 		usage(argv[0]);
 
-	if (!(flags & CLONE_NEWUSER) && uid != -1) {
-		ERROR("-u <uid> needs -s USER option");
+	if (!(flags & CLONE_NEWNET) && my_iflist) {
+		ERROR("-i <interfacename> needs -s NETWORK option");
+		return 1;
+	}
+
+	if (!(flags & CLONE_NEWUTS) && start_arg.want_hostname) {
+		ERROR("-H <hostname> needs -s UTSNAME option");
+		return 1;
+	}
+
+	if (!(flags & CLONE_NEWNS) && start_arg.want_default_mounts) {
+		ERROR("-M needs -s MOUNT option");
 		return 1;
 	}
 
 	pid = lxc_clone(do_start, &start_arg, flags);
 	if (pid < 0) {
 		ERROR("failed to clone");
-		return -1;
+		return 1;
 	}
 
+	if (my_iflist) {
+		for (tmpif = my_iflist; tmpif; tmpif = tmpif->mi_next) {
+			if (lxc_netdev_move_by_name(tmpif->mi_ifname, pid) < 0)
+				fprintf(stderr,"Could not move interface %s into container %d: %s\n", tmpif->mi_ifname, pid, strerror(errno));
+		}
+	}
+
+	if (daemonize)
+		exit(0);
+
 	if (waitpid(pid, &status, 0) < 0) {
 		ERROR("failed to wait for '%d'", pid);
-		return -1;
+		return 1;
 	}
 
 	return  lxc_error_set_and_log(pid, status);
