@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <unistd.h>
@@ -53,7 +53,7 @@ pid_t lxc_clone(int (*fn)(void *), void *arg, int flags)
 		.arg = arg,
 	};
 
-	long stack_size = sysconf(_SC_PAGESIZE);
+	size_t stack_size = sysconf(_SC_PAGESIZE);
 	void *stack = alloca(stack_size);
 	pid_t ret;
 
@@ -69,11 +69,11 @@ pid_t lxc_clone(int (*fn)(void *), void *arg, int flags)
 	return ret;
 }
 
-static char *namespaces_list[] = {
+static const char * const namespaces_list[] = {
 	"MOUNT", "PID", "UTSNAME", "IPC",
 	"USER", "NETWORK"
 };
-static int cloneflags_list[] = {
+static const int cloneflags_list[] = {
 	CLONE_NEWNS, CLONE_NEWPID, CLONE_NEWUTS, CLONE_NEWIPC,
 	CLONE_NEWUSER, CLONE_NEWNET
 };
