@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __parse_h
 #define __parse_h
@@ -28,16 +28,13 @@ typedef int (*lxc_dir_cb)(const char *name, const char *directory,
 
 typedef int (*lxc_file_cb)(char *buffer, void *data);
 
-extern int lxc_dir_for_each(const char *name, const char *directory,
-			    lxc_dir_cb callback, void *data);
-
 extern int lxc_file_for_each_line(const char *file, lxc_file_cb callback,
 				  void* data);
 
-extern int lxc_char_left_gc(char *buffer, size_t len);
+extern int lxc_char_left_gc(const char *buffer, size_t len);
 
-extern int lxc_char_right_gc(char *buffer, size_t len);
+extern int lxc_char_right_gc(const char *buffer, size_t len);
 
-extern int lxc_is_line_empty(char *line);
+extern int lxc_is_line_empty(const char *line);
 
 #endif
