@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <sys/types.h>
@@ -99,7 +99,10 @@ int lxc_sync_wake_child(struct lxc_handler *handler, int sequence)
 
 int lxc_sync_init(struct lxc_handler *handler)
 {
-	if (socketpair(AF_LOCAL, SOCK_STREAM, 0, handler->sv)) {
+	int ret;
+
+	ret = socketpair(AF_LOCAL, SOCK_STREAM, 0, handler->sv);
+	if (ret) {
 		SYSERROR("failed to create synchronization socketpair");
 		return -1;
 	}
