@@ -16,7 +16,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#include "../lxc/lxccontainer.h"
+#include <lxc/lxccontainer.h>
 
 #include <unistd.h>
 #include <signal.h>
@@ -25,7 +25,8 @@
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <lxc/state.h>
+#include <string.h>
+#include "lxc/state.h"
 
 #define MYNAME "lxctest1"
 
@@ -71,7 +72,7 @@ static int create_busybox(void)
 		return -1;
 	}
 	if (pid == 0) {
-		ret = execlp("lxc-create", "lxc-create", "-t", "busybox", "-f", "/etc/lxc/default.conf", "-n", MYNAME, NULL);
+		ret = execlp("lxc-create", "lxc-create", "-t", "busybox", "-n", MYNAME, NULL);
 		// Should not return
 		perror("execl");
 		exit(1);
@@ -218,14 +219,8 @@ int main(int argc, char *argv[])
 	}
 	free(sstr);
 
-	printf("hit return to start container");
-	char mychar;
-	ret = scanf("%c", &mychar);
-	if (ret < 0)
-		goto out;
-
 	/* non-daemonized is tested in 'startone' */
-	c->want_daemonize(c);
+	c->want_daemonize(c, true);
 	if (!c->startl(c, 0, NULL, NULL)) {
 		fprintf(stderr, "%d: %s failed to start daemonized\n", __LINE__, c->name);
 		goto out;
@@ -243,12 +238,6 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	printf("hit return to finish");
-	ret = scanf("%c", &mychar);
-	if (ret < 0)
-		goto out;
-
-
 	fprintf(stderr, "all lxc_container tests passed for %s\n", c->name);
 	ret = 0;
 
