@@ -16,7 +16,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#include "../lxc/lxccontainer.h"
+#include <lxc/lxccontainer.h>
 
 #include <unistd.h>
 #include <signal.h>
@@ -25,7 +25,8 @@
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <lxc/state.h>
+#include <string.h>
+#include "lxc/state.h"
 
 #define MYNAME "lxctest1"
 
@@ -170,8 +171,8 @@ int main(int argc, char *argv[])
 		ret = 1;
 		goto out;
 	}
-	if (!c->createl(c, "ubuntu", "-r", "lucid", NULL)) {
-		fprintf(stderr, "%d: failed to create a lucid container\n", __LINE__);
+	if (!c->createl(c, "busybox", NULL, NULL, 0, NULL)) {
+		fprintf(stderr, "%d: failed to create a trusty container\n", __LINE__);
 		ret = 1;
 		goto out;
 	}
@@ -280,22 +281,22 @@ int main(int argc, char *argv[])
 	printf("%d: get_config_item (cgroup.devices.devices.allow) returned %d %s\n", __LINE__, ret, v3);
 
 	if (!c->clear_config_item(c, "lxc.cgroup")) {
-		fprintf(stderr, "%d: failed clearing lxc.cgroup", __LINE__);
+		fprintf(stderr, "%d: failed clearing lxc.cgroup\n", __LINE__);
 		ret = 1;
 		goto out;
 	}
 	if (!c->clear_config_item(c, "lxc.cap.drop")) {
-		fprintf(stderr, "%d: failed clearing lxc.cap.drop", __LINE__);
+		fprintf(stderr, "%d: failed clearing lxc.cap.drop\n", __LINE__);
 		ret = 1;
 		goto out;
 	}
 	if (!c->clear_config_item(c, "lxc.mount.entries")) {
-		fprintf(stderr, "%d: failed clearing lxc.mount.entries", __LINE__);
+		fprintf(stderr, "%d: failed clearing lxc.mount.entries\n", __LINE__);
 		ret = 1;
 		goto out;
 	}
 	if (!c->clear_config_item(c, "lxc.hook")) {
-		fprintf(stderr, "%d: failed clearing lxc.hook", __LINE__);
+		fprintf(stderr, "%d: failed clearing lxc.hook\n", __LINE__);
 		ret = 1;
 		goto out;
 	}
