@@ -16,7 +16,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#include "../lxc/lxccontainer.h"
+#include <lxc/lxccontainer.h>
 
 #include <unistd.h>
 #include <signal.h>
@@ -24,11 +24,13 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #define MYNAME "lxctest1"
 
-static int destroy_ubuntu(void)
+static int destroy_container(void)
 {
 	int status, ret;
 	pid_t pid = fork();
@@ -60,7 +62,7 @@ again:
 	return WEXITSTATUS(status);
 }
 
-static int create_ubuntu(void)
+static int create_container(void)
 {
 	int status, ret;
 	pid_t pid = fork();
@@ -70,7 +72,7 @@ static int create_ubuntu(void)
 		return -1;
 	}
 	if (pid == 0) {
-		ret = execlp("lxc-create", "lxc-create", "-t", "ubuntu", "-f", "/etc/lxc/default.conf", "-n", MYNAME, NULL);
+		ret = execlp("lxc-create", "lxc-create", "-t", "busybox", "-n", MYNAME, NULL);
 		// Should not return
 		perror("execl");
 		exit(1);
@@ -115,9 +117,9 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	ret = create_ubuntu();
+	ret = create_container();
 	if (ret) {
-		fprintf(stderr, "%d: failed to create a ubuntu container\n", __LINE__);
+		fprintf(stderr, "%d: failed to create a container\n", __LINE__);
 		goto out;
 	}
 
@@ -157,27 +159,15 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	printf("hit return to start container");
-	char mychar;
-	ret = scanf("%c", &mychar);
-	if (ret < 0)
-		goto out;
-
 	if (!lxc_container_get(c)) {
 		fprintf(stderr, "%d: failed to get extra ref to container\n", __LINE__);
 		exit(1);
 	}
-	pid_t pid = fork();
-	if (pid < 0) {
-		fprintf(stderr, "%d: fork failed\n", __LINE__);
-		goto out;
-	}
-	if (pid == 0) {
-		b = c->startl(c, 0, NULL);
-		if (!b)
-			fprintf(stderr, "%d: %s failed to start\n", __LINE__, c->name);
-		lxc_container_put(c);
-		exit(!b);
+
+	c->want_daemonize(c, true);
+	if (!c->startl(c, 0, NULL)) {
+		fprintf(stderr, "%d: %s failed to start\n", __LINE__, c->name);
+		exit(1);
 	}
 
 	sleep(3);
@@ -194,7 +184,7 @@ int main(int argc, char *argv[])
 	}
 
 	len = c->get_cgroup_item(c, "cpuset.cpus", buf, 200);
-	if (len <= 0 || strcmp(buf, "0\n")) {
+	if (len <= 0 || strncmp(buf, "0", 1)) {
 		fprintf(stderr, "%d: not able to get cpuset.cpus (len %d buf %s)\n", __LINE__, len, buf);
 		goto out;
 	}
@@ -215,13 +205,10 @@ int main(int argc, char *argv[])
 
 	c->set_cgroup_item(c, "freezer.state", "THAWED");
 
-	printf("hit return to finish");
-	ret = scanf("%c", &mychar);
-	if (ret < 0)
-		goto out;
 	c->stop(c);
 
     /* feh - multilib has moved the lxc-init crap */
+#if 0
     goto ok;
 
 	ret = system("mkdir -p " LXCPATH "/lxctest1/rootfs//usr/local/libexec/lxc");
@@ -251,13 +238,14 @@ int main(int argc, char *argv[])
 	//  auto-check result?  ('bobo' is printed on stdout)
 
 ok:
+#endif
 	fprintf(stderr, "all lxc_container tests passed for %s\n", c->name);
 	ret = 0;
 
 out:
 	if (c) {
 		c->stop(c);
-		destroy_ubuntu();
+		destroy_container();
 	}
 	lxc_container_put(c);
 	exit(ret);
